DATA SEGMENT
    ; 定义与硬件寄存器相关的常量，使用EQU将硬件地址与符号名关联
    I8255_A EQU 210H        ; 8255 A口的地址
    I8255_C EQU 212H        ; 8255 C口的地址
    I8255_CTR EQU 213H      ; 8255 控制寄存器的地址

    I8254_0 EQU 220H        ; 8254计时器0的地址
    I8254_1 EQU 224H        ; 8254计时器1的地址
    I8254_CTR EQU 22CH      ; 8254计时器控制寄存器的地址

    I8259_0 EQU 200H        ; 8259中断控制器0的地址
    I8259_1 EQU 201H        ; 8259中断控制器1的地址

    LED_SEG EQU 230H        ; LED段控制寄存器的地址
    LED_BIT EQU 231H        ; LED位控制寄存器的地址

    COUNT DB ?              ; 存储计数值
    PORTCL DB 81H,41H,24H,22H,0FFH ; 定义一些端口数据
    LEDCODE DB 3FH,06H,5BH,4FH,66H,6DH,7DH,07H,7FH ; LED显示代码，代表数字0到9的七段显示编码
DATA ENDS

SSG SEGMENT STACK
    DW 100 DUP(0)          ; 定义堆栈段，分配256字节的空间
SSG ENDS

CODE SEGMENT
ASSUME CS:CODE, DS:DATA  ; 假设代码段为CS，数据段为DS

START: CLI                ; 禁用中断，防止在初始化期间发生中断
MOV AX, DATA              ; 将数据段地址加载到AX寄存器
MOV DS, AX                ; 将AX的值（数据段地址）加载到DS寄存器

; 初始化8254计时器
MOV DX, I8254_CTR         ; 选择8254控制寄存器
MOV AL, 00110111B         ; 设置计时器模式（定时模式设置）
OUT DX, AL                ; 将控制字写入控制寄存器
MOV DX, I8254_0           ; 选择计时器0
MOV AL, 0                 ; 将计时器0的计数值设为0
OUT DX, AL                ; 将值写入计时器0
OUT DX, AL                ; 再次写入，确保计时器复位
MOV DX, I8254_CTR         ; 选择8254控制寄存器
MOV AL, 01010110B         ;设置为分频模式（100Hz）
OUT DX, AL                ; 将控制字写入控制寄存器
MOV DX, I8254_1           ; 选择计时器1
MOV AL, 100               ; 将100Hz分频为1Hz
OUT DX, AL                ; 将值写入计时器1
MOV DX, I8255_CTR         ;
MOV AL, 90H               ; 设置8255控制字
OUT DX, AL                ; 写入控制字

; 初始化8259中断控制器
MOV DX, I8259_0           ; 选择8259中断控制器0
MOV AL, 13H               ; 设置中断向量
OUT DX, AL                ; 将设置值写入中断控制器0
MOV DX, I8259_1           ; 选择8259中断控制器1
MOV AL, 08H               ; 配置中断控制器1
OUT DX, AL                ; 将配置值写入中断控制器1
MOV AL, 01H               ; 使能中断
OUT DX, AL                ; 发送中断使能指令

; 设置中断服务例程
PUSH DS
MOV AX, 0
MOV DS, AX
LEA AX, CS:INT_PROC       ; 获取中断处理程序的地址
MOV SI, 08H
ADD SI, SI
ADD SI, SI
MOV DS:[SI], AX           ; 设置中断向量
PUSH CS
POP AX
MOV DS:[SI+2], AX
POP DS

; 启动中断
IN AL, DX                 ; 从端口读取数据
AND AL, 11111110B         ; 清除最低位
OUT DX, AL                ; 写回数据
STI                       ; 启用中断

RE_ON: MOV BX, 0           ; 设置BX为0

; 主循环
ON:
MOV AL, PORTCL[BX]        ; 从PORTCL中读取值
CMP AL, 0FFH              ; 比较值是否为FF
JZ RE_ON                  ; 如果是FF，返回RE_ON，重新开始
MOV DX, I8255_C           ; 设置I8255 C口
OUT DX, AL                ; 输出数据到I8255 C口

; 以下为对不同PORTCL值的处理逻辑，通过多个循环进行不同的操作，主要是LED显示相关

; 增加计数器，并通过计数器控制LED显示
INC BX
MOV COUNT, 8

LOOP1: CALL DISPLAY        ; 调用DISPLAY子程序，更新显示
MOV DX, I8255_A           ; 读取I8255 A口的状态
IN AL, DX
AND AL, 01H               ; 判断最低位是否为1
CMP AL, 1
JNZ LOOP1                 ; 如果最低位不为1，继续循环

LOOP2: MOV DX, I8255_A     ; 等待I8255 A口的状态变为0
IN AL, DX
AND AL, 01H
CMP AL, 0
JNZ LOOP2
DEC COUNT                 ; 计数器减1
CMP COUNT, 0              ; 判断是否为0
JNZ LOOP1                 ; 如果COUNT不为0，继续循环

; 对PORTCL中的下一个数据进行操作
MOV AL, PORTCL[BX]
CMP AL, 0FFH
JZ RE_ON                  ; 如果是FF，重新开始
MOV DX, I8255_C
OUT DX, AL

; 其他类似的循环...
INC BX
MOV COUNT, 2

LOOP3: CALL DISPLAY
MOV DX, I8255_A
IN AL, DX
AND AL, 01H
CMP AL, 1
JNZ LOOP3

LOOP4: MOV DX, I8255_A
IN AL, DX
AND AL, 01H
CMP AL, 0
JNZ LOOP4
DEC COUNT
CMP COUNT, 0
JNZ LOOP3

MOV AL, PORTCL[BX]
CMP AL, 0FFH
JZ RE_ON
MOV DX, I8255_C
OUT DX, AL

; 更多类似的操作，更新LED显示，控制I/O端口

INC BX
MOV COUNT, 5

LOOP5: CALL DISPLAY
MOV DX, I8255_A
IN AL, DX
AND AL, 01H
CMP AL, 1
JNZ LOOP5

LOOP6: MOV DX, I8255_A
IN AL, DX
AND AL, 01H
CMP AL, 0
JNZ LOOP6
DEC COUNT
CMP COUNT, 0
JNZ LOOP5

MOV AL, PORTCL[BX]
CMP AL, 0FFH
JZ RE_ON
MOV DX, I8255_C
OUT DX, AL

; 更多循环操作

INC BX
MOV COUNT, 2

LOOP7: CALL DISPLAY
MOV DX, I8255_A
IN AL, DX
AND AL, 01H
CMP AL, 1
JNZ LOOP7

LOOP8: MOV DX, I8255_A
IN AL, DX
AND AL, 01H
CMP AL, 0
JNZ LOOP8
DEC COUNT
CMP COUNT, 0
JNZ LOOP7

JMP ON                    ; 循环回到ON标签，重新开始

; DISPLAY子程序：更新LED显示
DISPLAY PROC
PUSH BX
MOV AL, COUNT
LEA SI, LEDCODE           ; 获取LED显示编码的起始地址
MOV BX, SI
AND AL, 0FH               ; 获取COUNT的低4位
MOV AH, 0
ADD SI, AX                ; 通过COUNT的低4位偏移LEDCODE数组
MOV AL, [SI]              ; 获取相应的显示编码
MOV DX, LED_SEG           ; 设置LED段控制寄存器
OUT DX, AL                ; 输出编码到LED显示
MOV AL, 01H               ; 启动显示
MOV DX, LED_BIT           ; 设置LED位控制寄存器
OUT DX, AL                ; 输出控制信号
CALL DELAY                ; 调用延时
POP BX
RET
DISPLAY ENDP

; DELAY子程序：简单的延时实现
DELAY PROC
    PUSH CX                  ; 保存CX寄存器的值
    MOV CX, 50H              ; 设置延迟循环次数（50H即80次循环）
KK:
    LOOP KK                  ; 循环，直到CX寄存器为0
    POP CX                   ; 恢复CX寄存器的值
    RET                       ; 返回调用点
DELAY ENDP

; INT_PROC: 中断处理程序
INT_PROC: 
    PUSH AX                  ; 保存AX寄存器的值
    PUSH BX                  ; 保存BX寄存器的值
    PUSH CX                  ; 保存CX寄存器的值
    PUSH DX                  ; 保存DX寄存器的值

    MOV AL, PORTCL[0]        ; 从PORTCL[0]读取一个值
    MOV DX, I8255_C          ; 设置8255控制寄存器C
    OUT DX, AL               ; 将AL的值输出到I8255 C口

    MOV DX, I8259_0          ; 设置8259中断控制器0
    MOV AL, 00100000B        ; 向中断控制器发送指令，告知处理了一个中断
    OUT DX, AL               ; 将指令发送给8259中断控制器

    POP DX                   ; 恢复DX寄存器的值
    POP CX                   ; 恢复CX寄存器的值
    POP BX                   ; 恢复BX寄存器的值
    POP AX                   ; 恢复AX寄存器的值

    STI                      ; 启用中断
    IRET                     ; 从中断返回